@@ -153,7 +153,9 @@
      VAR_PREFETCH = 370,
      VAR_PREFETCH_KEY = 371,
      VAR_SO_SNDBUF = 372,
-     VAR_HARDEN_BELOW_NXDOMAIN = 373
+     VAR_HARDEN_BELOW_NXDOMAIN = 373,
+     VAR_IGNORE_CD_FLAG = 374,
+     VAR_LOG_QUERIES = 375
    };
 #endif
 /* Tokens.  */
@@ -273,6 +275,8 @@
 #define VAR_PREFETCH_KEY 371
 #define VAR_SO_SNDBUF 372
 #define VAR_HARDEN_BELOW_NXDOMAIN 373
+#define VAR_IGNORE_CD_FLAG 374
+#define VAR_LOG_QUERIES 375
 
 
 
@@ -289,7 +293,7 @@ typedef union YYSTYPE
 
 
 /* Line 1685 of yacc.c  */
-#line 293 "util/configparser.h"
+#line 297 "util/configparser.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
